;  ---------==========----------==========---------=========---------
;      PROGRAM FOR REMOTE CONTROL FOR 8+2
;  ---------==========----------==========---------=========---------
 
$MOD51
 
INPUT   EQU     P3.2    ; Port3,Bit2 is used as input. The demodulated signal
             		           ; with active low level is connected to this pin
ZERO    EQU    P3.3  ; Zero Crossing Input

OUTPUT EQU P2
OP1	EQU	P2.2	
OP2	EQU	P2.3
OP3	EQU	P2.4	
OP4	EQU	P2.7	
OP5	EQU	P2.6	
OP6	EQU	P2.5	

FAN	EQU	P0.7	
FANL	EQU	P3.6
SWITCH 	EQU 	P3.7


OP7	EQU	P0.0	;A,7
OP8	EQU	P0.1	;B,6
OP9	EQU	P0.2	;C,4
OP10	EQU	P0.3	;D,2
OP11	EQU	P0.4	;E,1
OP12	EQU	P0.5	;F,9
OP13	EQU	P0.6	;G,10

SDA EQU P2.1  	;SDA=PIN5
SCL EQU P2.0	;SCL=PIN6
WTCMD   EQU     10100000B       ;WRITE DATA COMMAND Note 3
RDCMD   EQU     10100001B       ;READ DATA COMMAND Note 3


DSEG            ; This is internal data memory
ORG     20H     ; Bit adressable memory

FLAGS:  DS      1
CONTROL BIT     FLAGS.0  ; toggles with every new keystroke
NEW     BIT     FLAGS.1  ; Bit set when a new command has been received

STATUS:	DS	1		;Lights Status Bits
LIG1	BIT	STATUS.0
LIG2	BIT	STATUS.1
LIG3	BIT	STATUS.2
LIG4	BIT	STATUS.3
LIG5	BIT	STATUS.4
LIG6	BIT	STATUS.5
LIG7	BIT	STATUS.6		;FAN



COMMAND: DS     1       ; Received command byte
SUBAD:  DS      1       ; Device subaddress
BUFFER: DS      30      ; Buffer to store length of transmitted pulses
TOGGLE: DS     1	;Toggle every bit
ANS: DS	1	;
SPEED:	DS	1 ; Speed control Register
SPEED1:	DS	1
ADDR:	DS	1
STACK:  DS      1       ; Stack begins here
CSEG            ; Code begins here
 
;---------==========----------==========---------=========---------
;              PROCESSOR INTERRUPT AND RESET VECTORS
;---------==========----------==========---------=========---------
 
         ORG     00H    ; Reset
         JMP     MAIN
 
         ORG     0003H  ; External Interrupt0
         JMP     RECEIVE

         ORG 000BH        ;Timer Interrupt 0
          JMP TRIGGER

         ORG 0013H ; External Interrupt 1
         JMP SPED
 
                  
	 
;  ---------==========----------==========---------=========---------
;  Interrupt 0  routine
;  ---------==========----------==========---------=========---------
 
RECEIVE:
      	  MOV  2,#255               ; Time Loop (3/4 bit time) 
             DJNZ 2,$                  ; Waste Time to sync second bit 
                MOV  2,#255               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit 
	Mov  2,#145               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit               
	clr	a
	mov	r6,#07h

pol1:	mov	c,Input
	rlc	a
	Mov  2,#255               ; Waste time for next BIT 
             Djnz 2,$      
	Mov  2,#255               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit 
	Mov  2,#255               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit 
	Mov  2,#114               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit 
	djnz	r6,pol1
	MOV SUBAD,A
	
	mov	r6,#06h
pol2:	
	mov	c,Input
	rlc	a

	Mov  2,#255               ; Waste time for next BIT 
             Djnz 2,$      
	Mov  2,#255               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit 
	Mov  2,#255               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit 
	Mov  2,#114               ; Time Loop (3/4 bit time) 
             Djnz 2,$                  ; Waste Time to sync second bit 
	djnz	r6,pol2
	Mov  COMMAND,A             ; Save Command at IRData memory 
	

	MOV A,SUBAD
	MOV ADDR,A
	ANL A,#0FH
	MOV SUBAD,A
	CJNE A,#03H,ZXC1
	MOV A,COMMAND
	CPL A
	MOV COMMAND,A
	AJMP ASZ
ZXC1:	MOV A,SUBAD
	CJNE A,#00H,ANSS
	AJMP ASZ

ASZ:	MOV A,ADDR
	ANL A,#20H
	MOV TOGGLE,A
	CJNE A,ANS,ANSS
	AJMP WAR
ANSS:	JMP ANS1	
WAR:	
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!     	
	MOV A,COMMAND
;-------------------------------------------
	MOV R0,A
	XRL A,#01H		; device 1
	JNZ CH1
	CALL DEV1
	AJMP GO
CH1:	MOV A,R0
	XRL A,#02H		; device 2
	JNZ CH2
	CALL DEV2
	AJMP GO
CH2:	MOV A,R0
	XRL A,#03H		; device 3
	JNZ CH3
	CALL DEV3
	AJMP GO
CH3:	MOV A,R0
	XRL A,#04H		; device 4
	JNZ CH4
	CALL DEV4
	AJMP GO
CH4:	MOV A,R0
	XRL A,#05H		; device 5
	JNZ CH5
	CALL DEV5
	AJMP GO
CH5:	MOV A,R0
	XRL A,#06H		; device 6
	JNZ CH6
	CALL DEV6
	AJMP GO
CH6:	MOV A,R0
	XRL A,#0DH		; Fan1 on/off
	JNZ CH7
	CALL DEV7
	AJMP GO
CH7:	MOV A,R0
	XRL A,#0CH		; all devices off
	JNZ CH8
	CALL DEV8
	AJMP GO
CH8:	MOV A,R0
	XRL A,#10H		; Fan1 speed +
	JNZ CH9
	CALL DEV9
	AJMP GO
CH9:	MOV A,R0
	XRL A,#11H		; Fan1 speed -
	JNZ GO
	CALL DEV10
	AJMP GO
GO:
;***********************************************************
; This routine writes a byte of data to EEPROM
; The EEPROM address is assumed to be in R4.  See NOTE 3
; The DATA to be written is asumed to be in R1
;***********************************************************
	MOV     A,#WTCMD        ;LOAD WRITE COMMAND
      	  CALL    OUTS            ;SEND IT
     	   MOV     A,#05H            ;GET BYTE ADDRESS
    	    CALL    OUT             ;SEND IT
    	    MOV     A,STATUS            ;GET DATA
    	    CALL    OUT             ;SEND IT
    	    CALL    STOP            ;SEND STOP CONDITION
 ;*********************************************************** 
	MOV ANS,TOGGLE
	MOV A,ANS
	CPL ACC.5
	MOV ANS,A
	 SETB   NEW           ; Set flag to indicate the new command

;################################################################	
ANS1:        
	RETI
 ;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 	DEVICE DECODING
 ;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

DEV1:	CPL OP1			;Light 1
	JB OP1,DE1
	SETB LIG1
	RET
DE1:	CLR LIG1
	RET
;-------------------------------------------
DEV2:	CPL OP2			;Light 2
	JB OP2,DE2
	SETB LIG2
	RET
DE2:	CLR LIG2
	RET
;-------------------------------------------
DEV3:	CPL OP3			;Light 3
	JB OP3,DE3
	SETB LIG3
	RET
DE3:	CLR LIG3
	RET
;-------------------------------------------
DEV4:	CPL OP4			;Light 4
	JB OP4,DE4	
	SETB LIG4
	RET
DE4:	CLR LIG4
	RET
;-------------------------------------------
DEV5:	CPL OP5			;Light 5
	JB OP5,DE5
	SETB LIG5
	RET
DE5:	CLR LIG5
	RET
;-------------------------------------------
DEV6:	CPL OP6			;Light 6
	JB OP6,DE6
	SETB LIG6
	RET
DE6:	CLR LIG6
	RET
;-------------------------------------------
DEV7:	CPL LIG7			;Fan
	JNB LIG7,KLJ
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
;	SPEED RESTORATION
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
	MOV     A,#WTCMD        ;LOAD WRITE COMMAND TO SEND ADDRESS
        	CALL    OUTS            ;SEND IT
       	 MOV     A,#06H            ;GET LOW BYTE ADDRESS
       	 CALL    OUT             ;SEND IT
       	 CALL    CREAD           ;GET DATA BYTE
	MOV SPEED,R1
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
	CLR FANL
	RET
KLJ:	SETB FANL
	RET		; Fan off

;-------------------------------------------
DEV8:			;all off
	MOV STATUS,#00H
	MOV OUTPUT,#0FFH
	CLR TR0
	SETB FANL
	RET
;-------------------------------------------
DEV9:			; Increase Speed 
	MOV R2,SPEED
	MOV A,#08H		;Chk If > than 08
	CLR C
	SUBB A,R2
	JNC DFDF
	MOV SPEED,#00H
DFDF:	CJNE R2,#08,DFF
	RET
DFF:	INC SPEED
;***********************************************************
	MOV     A,#WTCMD        ;LOAD WRITE COMMAND
      	  CALL    OUTS            ;SEND IT
     	   MOV     A,#06H            ;GET BYTE ADDRESS
    	    CALL    OUT             ;SEND IT
    	    MOV     A,SPEED            ;GET DATA
    	    CALL    OUT             ;SEND IT
    	    CALL    STOP            ;SEND STOP CONDITION
 ;*********************************************************** 
	CALL DISP
	RET
;-------------------------------------------
DEV10:			;Decrease Speed
	MOV R2,SPEED
	MOV A,#08H
	CLR C
	SUBB A,R2
	JNC DYDF
	MOV SPEED,#01H
DYDF:	CJNE R2,#00,DFG
	RET
DFG:	DEC SPEED
;***********************************************************
	MOV     A,#WTCMD        ;LOAD WRITE COMMAND
      	  CALL    OUTS            ;SEND IT
     	   MOV     A,#06H            ;GET BYTE ADDRESS
    	    CALL    OUT             ;SEND IT
    	    MOV     A,SPEED            ;GET DATA
    	    CALL    OUT             ;SEND IT
    	    CALL    STOP            ;SEND STOP CONDITION
 ;*********************************************************** 
	CALL DISP
	RET

;  ---------==========----------==========---------=========---------
;  Interrupt 1  routine
;  ---------==========----------==========---------=========---------
SPED:	
	SETB FAN
	JB LIG7,QWZ
	CLR TR0
	RETI
QWZ:	MOV R2,SPEED
	CJNE R2,#00H,QW1		; Fan Off
	SETB FAN
	CLR TR0
	RETI

QW1:	CJNE R2,#01H,QW2		;Fan Speed 1
	MOV TL0,#011H
	MOV TH0,#0F0H
	SETB TR0
	RETI

QW2:	CJNE R2,#02H,QW3		;Fan Speed 2
	MOV TL0,#0FFH
	MOV TH0,#0F0H
	SETB TR0
	RETI

QW3:	CJNE R2,#03H,QW4		;Fan Speed 3
	MOV TL0,#0FFH
	MOV TH0,#0F4H
	SETB TR0
	RETI

QW4:	CJNE R2,#04H,QW5		;Fan Speed 4
	MOV TL0,#0FFH
	MOV TH0,#0F6H
	SETB TR0
	RETI
QW5:	CJNE R2,#05H,QW6		;Fan Speed 5
	MOV TL0,#0FFH
	MOV TH0,#0F7H
	SETB TR0
	RETI
QW6:	CJNE R2,#06H,QW7		;Fan Speed 6
	MOV TL0,#0FFH
	MOV TH0,#0F9H
	SETB TR0
	RETI
QW7:	CJNE R2,#07H,QW8		;Fan Speed 7
	MOV TL0,#0FFH
	MOV TH0,#0FAH
	SETB TR0
	RETI
QW8:	CJNE R2,#08H,QW9		;Fan full Speed
	CLR TR0
	CLR FAN
QW9:	RETI
	
	


;  ---------==========----------==========---------=========---------
;  Timer 0  routine
;  ---------==========----------==========---------=========---------
TRIGGER:
	CLR FAN
	CLR TR0
	RETI


;  ---------==========----------==========---------=========---------
;  Main routine. Program execution starts here.
;  ---------==========----------==========---------=========---------
MAIN:    
	MOV SP,#60H
	MOV OUTPUT,#0FFH	;Switch off all devices
	MOV STATUS,#00H		;Clear all devices bits
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
;	RESTORATION
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
	SETB SWITCH
	JB SWITCH,NMB
	MOV     A,#WTCMD        ;LOAD WRITE COMMAND TO SEND ADDRESS
        CALL    OUTS            ;SEND IT
       	MOV     A,#05H            ;GET LOW BYTE ADDRESS
       	CALL    OUT             ;SEND IT
       	CALL    CREAD           ;GET DATA BYTE
	MOV STATUS,R1
	CALL RESTORE
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
;	SPEED RESTORATION
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
NMB:	MOV     A,#WTCMD        ;LOAD WRITE COMMAND TO SEND ADDRESS
        CALL    OUTS            ;SEND IT
       	MOV     A,#06H            ;GET LOW BYTE ADDRESS
       	CALL    OUT             ;SEND IT
       	CALL    CREAD           ;GET DATA BYTE
	MOV SPEED,R1
	CALL DISP
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

      	SETB   EX0          		 ; Enable external Interrupt0
        	CLR    IT0          		 ; triggered by a high to low transition
	SETB EX1
	CLR    IT1          		 ; triggered by a high to low transition
	MOV TMOD,#11H
	SETB ET0
	SETB ET1
	SETB   EA

         	MOV ANS,#00H		;clear temp toggle bit
    	
	CLR   NEW
	
	
LOO:	
	JNB NEW,LOO
	CLR NEW
	AJMP LOO

;&&&&&&&&&&&&&&&&&&&&&&&&&&&&
; 	POWER RESTORE
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&
RESTORE:
	JB STATUS.0,UTR1
	SETB OP1
	AJMP MN1
UTR1:	CLR OP1
MN1:	JB STATUS.1,UTR2
	SETB OP2
	AJMP MN2
UTR2:	CLR OP2
MN2:	JB STATUS.2,UTR3
	SETB OP3
	AJMP MN3
UTR3:	CLR OP3
MN3:	JB STATUS.3,UTR4
	SETB OP4
	AJMP MN4
UTR4:	CLR OP4
MN4:	JB STATUS.4,UTR5
	SETB OP5
	AJMP MN5
UTR5:	CLR OP5
MN5:	JB STATUS.5,UTR6
	SETB OP6
	AJMP MN6
UTR6:	CLR OP6
MN6:	JB STATUS.6,UTR7
	SETB FANL
	AJMP MN7
UTR7:	CLR FANL
MN7:	RET	

;&&&&&&&&&&&&&&&&&&&&&&&&&&&&
; 	7 SEGMENT DISPLAY
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&
DISP:
	MOV R2,SPEED
	CJNE R2,#00H,AAS1
	CLR OP7
	CLR OP8
	CLR OP9
	CLR OP10
	CLR OP11
	CLR OP12
	SETB OP13
	RET
AAS1:	MOV R2,SPEED
	CJNE R2,#01H,AS2
	CLR OP8
	CLR OP9
	SETB OP7
	SETB OP10
	SETB OP11
	SETB OP12
	SETB OP13
	RET
AS2:	MOV R2,SPEED
	CJNE R2,#02H,AS3
	CLR OP7
	CLR OP8
	CLR OP10
	CLR OP11
	CLR OP13
	SETB OP9
	SETB OP12
	RET
AS3:	MOV R2,SPEED
	CJNE R2,#03H,AS4
	CLR OP7
	CLR OP8
	CLR OP9
	CLR OP10
	CLR OP13
	SETB OP11
	SETB OP12
	RET
AS4:	MOV R2,SPEED
	CJNE R2,#04H,AS5
	CLR OP8
	CLR OP9
	CLR OP12
	CLR OP13
	SETB OP7
	SETB OP10
	SETB OP11
	RET
AS5:	MOV R2,SPEED
	CJNE R2,#05H,AS6
	CLR OP7
	CLR OP9
	CLR OP10
	CLR OP12
	CLR OP13
	SETB OP8
	SETB OP11
	RET
AS6:	MOV R2,SPEED
	CJNE R2,#06H,AS7
	CLR OP7
	CLR OP9
	CLR OP10
	CLR OP11
	CLR OP12
	CLR OP13
	SETB OP8
	RET
AS7:	MOV R2,SPEED
	CJNE R2,#07H,AS8
	CLR OP7
	CLR OP8
	CLR OP9
	SETB OP10
	SETB OP11
	SETB OP12
	SETB OP13
	RET
AS8:	MOV R2,SPEED
	CJNE R2,#08H,AS9
	CLR OP7
	CLR OP8
	CLR OP9
	CLR OP10
	CLR OP11
	CLR OP12
	CLR OP13
AS9:	RET
	


 ;***********************************************************************
; THIS ROUTINE SENDS OUT CONTENTS OF THE ACCUMULATOR
; to the EEPROM and includes START condition.  Refer to the data sheets
; for discussion of START and STOP conditions.
;***********************************************************************

OUTS:   MOV     R2,#8           ;LOOP COUNT -- EQUAL TO BIT COUNT
        SETB   SDA            ;INSURE DATA IS HI               
        SETB    SCL            ;INSURE CLOCK IS HI
        NOP                     ;NOTE 1
        NOP                     
        NOP
        CLR     SDA            ;START CONDITION -- DATA = 0
        NOP                     ;NOTE 1
        NOP      
        NOP
          CLR     SCL            ;CLOCK = 0
OTSLP:  RLC     A               ;SHIFT BIT
        JNC     BITLS
        SETB    SDA            ;DATA = 1
        JMP     OTSL1           ;CONTINUE
BITLS:  CLR     SDA            ;DATA = 0
OTSL1:  SETB    SCL            ;CLOCK HI
        NOP                     ;NOTE 1
        NOP
        NOP
        
        CLR     SCL            ;CLOCK LOW
        DJNZ    R2,OTSLP        ;DECREMENT COUNTER
        SETB    SDA            ;TURN PIN INTO INPUT
        NOP                     ;NOTE 1
                             
        SETB    SCL            ;CLOCK ACK
        NOP                     ;NOTE 1
        NOP
        NOP
      
        CLR     SCL
        RET

;**********************************************************************
; THIS ROUTINE SENDS OUT CONTENTS OF ACCUMLATOR TO EEPROM
; without sending a START condition.
;**********************************************************************

OUT:    MOV     R2,#8           ;LOOP COUNT -- EQUAL TO BIT COUNT
OTLP:   RLC     A               ;SHIFT BIT
        JNC     BITL            
        SETB    SDA            ;DATA = 1
        JMP     OTL1            ;CONTINUE
BITL:   CLR     SDA            ;DATA = 0
OTL1:   SETB    SCL            ;CLOCK HI
        NOP                     ;NOTE 1
        NOP
        NOP
     
        CLR     SCL            ;CLOCK LOW
        DJNZ    R2,OTLP         ;DECREMENT COUNTER
        SETB    SDA            ;TURN PIN INTO INPUT
        NOP                     ;NOTE 1
     
        SETB    SCL            ;CLOCK ACK
        NOP                     ;NOTE 1
        NOP
        NOP
      
        CLR     SCL
        RET


STOP:   CLR     SDA            ;STOP CONDITION SET DATA LOW
        NOP                     ;NOTE 1
        NOP
        NOP
      
        SETB    SCL            ;SET CLOCK HI
        NOP                     ;NOTE 1
        NOP
        NOP
      
        SETB    SDA            ;SET DATA HIGH
        RET
        

;*******************************************************************
; THIS ROUTINE READS A BYTE OF DATA FROM EEPROM
; From EEPROM current address pointer.
; Returns the data byte in R1
;*******************************************************************
CREAD:  MOV     A,#RDCMD        ;LOAD READ COMMAND
        CALL    OUTS            ;SEND IT
        CALL    IN              ;READ DATA
        MOV     R1,A            ;STORE DATA
        CALL    STOP            ;SEND STOP CONDITION
        RET

;**********************************************************************
; THIS ROUTINE READS IN A BYTE FROM THE EEPROM
; and stores it in the accumulator
;**********************************************************************

IN:     MOV     R2,#8           ;LOOP COUNT
        SETB    SDA            ;SET DATA BIT HIGH FOR INPUT
INLP:   CLR     SCL            ;CLOCK LOW
        NOP                     ;NOTE 1
        NOP
        NOP
        NOP
     
        SETB    SCL            ;CLOCK HIGH
        CLR     C               ;CLEAR CARRY
        JNB     SDA,INL1       ;JUMP IF DATA = 0
        CPL     C               ;SET CARRY IF DATA = 1
INL1:   RLC     A               ;ROTATE DATA INTO ACCUMULATOR
        DJNZ    R2,INLP         ;DECREMENT COUNTER
        CLR     SCL            ;CLOCK LOW
        RET

;*********************************************************************
; This routine test for WRITE DONE condition 
; by testing for an ACK.
; This routine can be run as soon as a STOP condition
; has been generated after the last data byte has been sent
; to the EEPROM. The routine loops until an ACK is received from 
; the EEPROM. No ACK will be received until the EEPROM is done with
; the write operation.
;*********************************************************************
ACKTST: MOV     A,#WTCMD        ;LOAD WRITE COMMAND TO SEND ADDRESS
        MOV     R2,#8           ;LOOP COUNT -- EQUAL TO BIT COUNT
        CLR     SDA            ;START CONDITION -- DATA = 0
        NOP                     ;NOTE 1
        NOP
        NOP
           
        CLR     SCL            ;CLOCK = 0
AKTLP:  RLC     A               ;SHIFT BIT
        JNC     AKTLS
        SETB    SDA            ;DATA = 1
        JMP     AKTL1           ;CONTINUE
AKTLS:  CLR     SDA            ;DATA = 0
AKTL1:  SETB    SCL            ;CLOCK HI
        NOP                     ;NOTE 1
        NOP
        NOP
      
        CLR     SCL            ;CLOCK LOW
        DJNZ    R2,AKTLP        ;DECREMENT COUNTER
        SETB    SDA            ;TURN PIN INTO INPUT
        NOP                     ;NOTE 1
      
        SETB    SCL            ;CLOCK ACK
        NOP                     ;NOTE 1
        NOP
        NOP
       
        JNB     SDA,EXIT       ;EXIT IF ACK (WRITE DONE)
        JMP     ACKTST          ;START OVER
EXIT:   CLR     SCL            ;CLOCK LOW
        CLR     SDA            ;DATA LOW
        NOP                     ;NOTE 1
        NOP
        NOP
      
        SETB    SCL            ;CLOCK HIGH
        NOP
        NOP
        SETB    SDA            ;STOP CONDITION
        RET


        END

